require "import"
import "android.widget.*"
import "android.view.*"
import "android.util.Base64"
import "android.graphics.Color"
import "android.content.ClipboardManager"
import "android.content.ClipData"
import "java.security.MessageDigest"
import "javax.crypto.Cipher"
import "javax.crypto.spec.SecretKeySpec"
import "android.graphics.drawable.GradientDrawable"
import "android.content.Context"
import "android.content.Intent"
import "android.app.AlertDialog"

-- تحويل dp إلى pixels
function dp2px(dp)
  return dp * activity.Resources.DisplayMetrics.density
end

activity.setTitle("محرر التشفير المتقدم")
activity.setTheme(android.R.style.Theme_Material_Light)

-- الألوان
colors = {
  primary = 0xFF2196F3,
  accent = 0xFF1976D2,
  text_primary = 0xFF212121,
  text_secondary = 0xFF757575,
  white = 0xFFFFFFFF,
  error = 0xFFD32F2F,
  background = 0xFFF5F5F5,
  exit_button = 0xFFF44336
}

-- إنشاء خلفية للأزرار
function createButtonBackground(color)
  local drawable = GradientDrawable()
  drawable.setColor(color)
  drawable.setCornerRadius(dp2px(8))
  return drawable
end

-- أنماط التشفير
encryption_methods = {
  "Base64",
  "MD5",
  "SHA-1",
  "SHA-256",
  "AES",
  "ROT13"
}

-- إنشاء محول للسبينر
methodAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, encryption_methods)
methodAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)

layout = {
  ScrollView,
  layout_width = "match_parent",
  layout_height = "match_parent",
  backgroundColor = colors.background,
  {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
      CardView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      cardElevation = "4dp",
      radius = "8dp",
      layout_marginBottom = "8dp",
      {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        {
          TextView,
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "النص المدخل",
          textSize = "18sp",
          textColor = colors.text_primary,
        },
        {
          EditText,
          id = "inputText",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          hint = "اكتب النص هنا",
          textSize = "16sp",
          padding = "8dp",
          minLines = 3,
        },
      },
    },
    {
      CardView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      cardElevation = "4dp",
      radius = "8dp",
      layout_marginBottom = "8dp",
      {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        {
          TextView,
          text = "خيارات التشفير",
          textSize = "18sp",
          textColor = colors.text_primary,
        },
        {
          Spinner,
          id = "methodSpinner",
          layout_width = "match_parent",
          layout_height = "wrap_content",
        },
        {
          EditText,
          id = "keyInput",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          hint = "مفتاح التشفير (مطلوب لـ AES)",
          visibility = View.GONE,
        },
      },
    },
    {
      LinearLayout,
      orientation = "horizontal",
      gravity = "center",
      {
        Button,
        id = "encodeButton",
        layout_width = "wrap_content",
        layout_height = "wrap_content",
        text = "تشفير",
      },
      {
        Button,
        id = "decodeButton",
        layout_width = "wrap_content",
        layout_height = "wrap_content",
        text = "فك التشفير",
      },
      {
        Button,
        id = "clearButton",
        layout_width = "wrap_content",
        layout_height = "wrap_content",
        text = "مسح",
      },
    },
    {
      CardView,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      cardElevation = "4dp",
      radius = "8dp",
      layout_marginTop = "8dp",
      {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        {
          TextView,
          text = "النتيجة",
          textSize = "18sp",
          textColor = colors.text_primary,
        },
        {
          TextView,
          id = "resultText",
          layout_width = "match_parent",
          layout_height = "wrap_content",
          text = "النتيجة ستظهر هنا",
          textSize = "16sp",
          padding = "8dp",
          textIsSelectable = true,
        },
        {
          LinearLayout,
          orientation = "horizontal",
          gravity = "center",
          {
            Button,
            id = "copyButton",
            layout_width = "wrap_content",
            layout_height = "wrap_content",
            text = "نسخ",
          },
          {
            Button,
            id = "shareButton",
            layout_width = "wrap_content",
            layout_height = "wrap_content",
            text = "مشاركة",
          },
        },
      },
    },
    {
      Button,
      id = "exitButton",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      text = "خروج",
      layout_marginTop = "16dp",
    },
  },
}

activity.setContentView(loadlayout(layout))

-- تعيين محول السبينر
methodSpinner.setAdapter(methodAdapter)

-- وظائف التشفير
function encodeBase64(text)
  return Base64.encodeToString(String(text).getBytes(), Base64.DEFAULT)
end

function decodeBase64(encodedText)
  local decodedBytes = Base64.decode(encodedText, Base64.DEFAULT)
  return String(decodedBytes)
end

function getMD5(text)
  local digest = MessageDigest.getInstance("MD5")
  local hash = digest.digest(text:getBytes())
  return toHexString(hash)
end

function getSHA1(text)
  local digest = MessageDigest.getInstance("SHA-1")
  local hash = digest.digest(text:getBytes())
  return toHexString(hash)
end

function getSHA256(text)
  local digest = MessageDigest.getInstance("SHA-256")
  local hash = digest.digest(text:getBytes())
  return toHexString(hash)
end

function rot13(text)
  local result = ""
  for i = 1, #text do
    local c = text:sub(i,i)
    local b = string.byte(c)
    if b >= 65 and b <= 90 then
      result = result .. string.char(((b - 65 + 13) % 26) + 65)
    elseif b >= 97 and b <= 122 then
      result = result .. string.char(((b - 97 + 13) % 26) + 97)
    else
      result = result .. c
    end
  end
  return result
end

function toHexString(bytes)
  local hex = ""
  for i = 0, #bytes-1 do
    hex = hex .. String.format("%02x", bytes[i] & 0xff)
  end
  return hex
end

-- معالجة تغيير طريقة التشفير
methodSpinner.onItemSelected = function(parent, view, position, id)
  if encryption_methods[position+1] == "AES" then
    keyInput.setVisibility(View.VISIBLE)
  else
    keyInput.setVisibility(View.GONE)
  end
end

-- زر التشفير
encodeButton.onClick = function()
  local input = tostring(inputText.getText())
  if input == "" then
    Toast.makeText(activity, "الرجاء إدخال نص!", Toast.LENGTH_SHORT).show()
    return
  end
  
  local method = tostring(methodSpinner.getSelectedItem())
  
  pcall(function()
    local result = ""
    if method == "Base64" then
      result = encodeBase64(input)
    elseif method == "MD5" then
      result = getMD5(input)
    elseif method == "SHA-1" then
      result = getSHA1(input)
    elseif method == "SHA-256" then
      result = getSHA256(input)
    elseif method == "ROT13" then
      result = rot13(input)
    elseif method == "AES" then
      local key = tostring(keyInput.getText())
      if key == "" then
        Toast.makeText(activity, "الرجاء إدخال مفتاح التشفير!", Toast.LENGTH_SHORT).show()
        return
      end
      result = encryptAES(input, key)
    end
    resultText.setText(result)
  end)
end

-- زر فك التشفير
decodeButton.onClick = function()
  local input = tostring(inputText.getText())
  if input == "" then
    Toast.makeText(activity, "الرجاء إدخال نص!", Toast.LENGTH_SHORT).show()
    return
  end
  
  local method = tostring(methodSpinner.getSelectedItem())
  
  pcall(function()
    if method == "Base64" then
      local decoded = decodeBase64(input)
      resultText.setText(decoded)
    elseif method == "ROT13" then
      resultText.setText(rot13(input))
    elseif method == "AES" then
      local key = tostring(keyInput.getText())
      if key == "" then
        Toast.makeText(activity, "الرجاء إدخال مفتاح التشفير!", Toast.LENGTH_SHORT).show()
        return
      end
      local decrypted = decryptAES(input, key)
      resultText.setText(decrypted)
    else
      Toast.makeText(activity, "لا يمكن فك تشفير " .. method, Toast.LENGTH_SHORT).show()
    end
  end)
end

-- زر المسح
clearButton.onClick = function()
  inputText.setText("")
  resultText.setText("النتيجة ستظهر هنا")
  keyInput.setText("")
  Toast.makeText(activity, "تم مسح جميع الحقول", Toast.LENGTH_SHORT).show()
end

-- زر النسخ
copyButton.onClick = function()
  local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
  local clip = ClipData.newPlainText("نتيجة التشفير", resultText.getText())
  clipboard.setPrimaryClip(clip)
  Toast.makeText(activity, "تم نسخ النص!", Toast.LENGTH_SHORT).show()
end

-- زر المشاركة
shareButton.onClick = function()
  local intent = Intent(Intent.ACTION_SEND)
  intent.setType("text/plain")
  intent.putExtra(Intent.EXTRA_TEXT, tostring(resultText.getText()))
  activity.startActivity(Intent.createChooser(intent, "مشاركة عبر"))
end

-- زر الخروج
exitButton.onClick = function()
  local builder = AlertDialog.Builder(activity)
  builder.setTitle("تأكيد الخروج")
  builder.setMessage("هل تريد الخروج من التطبيق؟")
  builder.setPositiveButton("نعم", {onClick = function()
    activity.finish()
  end})
  builder.setNegativeButton("لا", nil)
  builder.show()
end

-- تطبيق الألوان على الأزرار
encodeButton.setBackgroundColor(colors.primary)
decodeButton.setBackgroundColor(colors.primary)
clearButton.setBackgroundColor(colors.primary)
copyButton.setBackgroundColor(colors.primary)
shareButton.setBackgroundColor(colors.primary)
exitButton.setBackgroundColor(colors.exit_button)

-- تعيين لون النص للأزرار
encodeButton.setTextColor(colors.white)
decodeButton.setTextColor(colors.white)
clearButton.setTextColor(colors.white)
copyButton.setTextColor(colors.white)
shareButton.setTextColor(colors.white)
exitButton.setTextColor(colors.white)